@@ -1,4 +1,4 @@
-#	$OpenBSD: putty-ciphers.sh,v 1.6 2017/05/08 01:52:49 djm Exp $
+#	$OpenBSD: putty-ciphers.sh,v 1.7 2020/01/23 03:35:07 dtucker Exp $
 #	Placed in the Public Domain.
 
 tid="putty ciphers"
@@ -8,14 +8,14 @@ if test "x$REGRESS_INTEROP_PUTTY" != "xyes" ; then
 	exit 0
 fi
 
-for c in aes 3des aes128-ctr aes192-ctr aes256-ctr ; do
+for c in aes 3des aes128-ctr aes192-ctr aes256-ctr chacha20 ; do
 	verbose "$tid: cipher $c"
 	cp ${OBJ}/.putty/sessions/localhost_proxy \
 	    ${OBJ}/.putty/sessions/cipher_$c
 	echo "Cipher=$c" >> ${OBJ}/.putty/sessions/cipher_$c
 
 	rm -f ${COPY}
-	env HOME=$PWD ${PLINK} -load cipher_$c -batch -i putty.rsa2 \
+	env HOME=$PWD ${PLINK} -load cipher_$c -batch -i ${OBJ}/putty.rsa2 \
 	    cat ${DATA} > ${COPY}
 	if [ $? -ne 0 ]; then
 		fail "ssh cat $DATA failed"
