@@ -1,4 +1,4 @@
-#	$OpenBSD: sshsig.sh,v 1.3 2019/11/26 23:43:10 djm Exp $
+#	$OpenBSD: sshsig.sh,v 1.4 2020/03/13 03:18:45 djm Exp $
 #	Placed in the Public Domain.
 
 tid="sshsig"
@@ -133,7 +133,7 @@ for t in $SIGNKEYS; do
 	# check-novalidate with invalid data
 	${SSHKEYGEN} -vvv -Y check-novalidate -s $sigfile -n $sig_namespace \
 		< $DATA2 >/dev/null 2>&1 && \
-		fail "sucessfully checked signature for $t key with invalid data"
+		fail "succeeded checking signature for $t key with invalid data"
 
 	# Check signing keys using ssh-agent.
 	${SSHADD} -D >/dev/null 2>&1 # Remove all previously-loaded keys.
